<?php
/**
 * @Copyright
 *
 * @package js_weather for Joomla! 4.x
 * @author     Joachim Schmidt {@link http://www.jschmidt-systemberatung.de/}
 * @version Version: 1.0.0 - 15-February-2023
 * @link       Project Site {@link http://www.jschmidt-systemberatung.de/}
 *
 * @license GNU/GPL
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * change activity:
 *  24.02.2023 First build (v1.0.0)
 *  10.10.2023: changed code to avoid php 8.2 warmimgs (deprecated: Creation of dynamic property)
 *  27.03.2024 add option to provide country code for locations
 *  30.03.2024 improve code for providing error messages
 */
namespace js_weather\module;
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;

class js_weatherClass
{
	var $api_key;
	var $url;
	var $units;
	var $lang;
	var $tag;
	var $id;
	var $location;
	var $country;
	var $use_ip;
	var $beautify_json;
	
	function __construct ($api_key, $forecast = false)
	{
		$this->js_weather($api_key, $forecast);
	}

	function js_weather ($api_key, $forecast)
	{
		$this->api_key = $api_key;
		if ($forecast)
			$this->url = "https://api.openweathermap.org/data/2.5/forecast?";
		else
			$this->url = "https://api.openweathermap.org/data/2.5/weather?";
		$this->units = "metric";
		$this->lang = 'de';
		$this->tag = '';
		$this->id = 1;
		$this->location = "q=Frankfurt";
		$this->country = "de";
		$this->use_ip = false;
		$this->beautify_json = false;
	}

	function setLocation ($location, $use_ip = false, $country = "" )
	{
		if ($use_ip)
		{
			$ip = $this->getIp();

			if ($ip)
				$ip_info = $this->getIpinfo($ip);
			else
				$this->use_ip = false;
	
			if ($ip_info == null)
			{
				$this->use_ip = false;
			}
			else
			{
				$this->location = "lat=" . $ip_info['city']['lat'];
				$this->location .= "&lon=" . $ip_info['city']['lon'];
				$this->tag = $ip_info['city']['tag'];
				$this->use_ip = true;
				return true;
			}
		}
		
		if ( strpos($location, ",") )
		{
			$tmp = explode("," , $location);
			if ( is_numeric($tmp[0]) && is_numeric($tmp[1]) )
			{
				$this->location = "lat=" . $tmp[0];
				$this->location .= "&lon=" . $tmp[1];
				return true;
			}
			else
			{
			  return false;
			}
		}
		
		if ($country != "")
			$this->location = "q=" . $location . "," . substr($country, 0, 2);
		else
		   $this->location = "q=" . $location;
		return true;
	}

	function getUnitInfo ($type = "metric")
	{
				
		switch  ($type)
		{
			case "metric":
				$info = [
				 'temperature_unit' => "C",
				 'speed_unit' => " km/h ",
				 'visibilty_unit' => " m"
		        ];
				break;
				
			case "imperial":
				$info = [
				'temperature_unit' => "F",
				'speed_unit' => " mph ",
				'visibilty_unit' => " ft"
				];
				break;
				
			default:
				$info = [
				'temperature_unit' => "K",
				'speed_unit' => " km/h ",
				'visibilty_unit' => " m"
				];
		}
		
		return $info;
	}
	
	function getLocation ()
	{
		return $this->location;
	}
	
	function setCounty ($ountry)
	{
		$this->country = substr($country, 0, 2);
	}
	
	function getCounty ()
	{
		return $this->country;
	}
	
	function getUseIP ()
	{
		return $this->use_ip;
	}
	
	function setUseIP ($use = true)
	{
		$this->use_ip = $use;
	}
	
	function getTag ()
	{
		return $this->tag;
	}

	function setLanguage ($tag)
	{
		$this->lang = $tag;
	}

	function getLanguage ()
	{
		return $this->lang;
	}
	
	function setID ($id)
	{
		$this->id = $id;
	}
	
	function getID ()
	{
		return $this->id;
	}

	function setUnits ($units)
	{
		if ($units == "1" || $units == 'metric')
			$this->units = 'metric';
		elseif ($units == "2" || $units == 'imperial')
			$this->units = 'imperial';
		else
			$this->units = 'standard';
	}

	function getunits ()
	{
		return $this->units;
	}

	function get_weather_data ($dir = null)
	{
		$location =  str_replace ( " ", "+", trim( $this->getlocation() ) );
		
		$url = $this->url . $location . "&lang=" . $this->lang . "&units=" . $this->units;
		$url .= "&appid=" . $this->api_key;

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 0);
		curl_setopt($ch, CURLOPT_TIMEOUT, 3);
		curl_setopt($ch, CURLOPT_VERBOSE, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		$response = curl_exec($ch);

		if (! $response)
		{
			$error_info = '{ "cod": "curl", "message": "' . curl_error($ch) . '<br>url = ' . $url   . '"}';
			curl_close($ch);
			return $error_info;
		}

		curl_close($ch);

		$weather_data = json_decode($response);
		if (json_last_error())
		{
			$tmp = array(
					null,
					null
			);
			if (strpos($response, "</title>"))
			{
				$tmp = explode("</title>", $response);
				$tmp[1] = str_replace("<center>", "<center> ", $tmp[1]);
			}
			else
				$tmp[1] = $response;

			$response = strip_tags($tmp[1]);
			$response = preg_replace('/[[:cntrl:]]/', '', trim($response));
			$response = '{ "cod":"' . substr($response, 0, 3) . '","message":"' . substr($response, 3) . '"}';

			return $response;
		}
		

		if ($dir && $weather_data->cod == 200)
			$this->save_weather_data($dir, $response);

		return $response;
	}

	function save_weather_data ($dir, $data)
	{
		if (file_exists($dir))
		{
			$file = $dir . "/save_weatherdata." . $this->id;
			if ($this->beautify_json)
				$data = $this->toPrettyString($data);
			$fp = fopen($file, "w");
			fwrite($fp, $data);
		}
	}

	function getIp ()
	{
		// return "109.234.162.136";
		$keys = array(
				'HTTP_CLIENT_IP',
				'HTTP_X_FORWARDED_FOR',
				'HTTP_X_FORWARDED',
				'HTTP_X_CLUSTER_CLIENT_IP',
				'HTTP_FORWARDED_FOR',
				'HTTP_FORWARDED',
				'REMOTE_ADDR'
		);
		foreach ($keys as $k)
		{
			if (isset($_SERVER[$k]) && strpos($_SERVER[$k], ',') !== false)
			{
				$ips = explode(',', $_SERVER[$k]);
				$_SERVER[$k] = trim($ips[count($ips) - 1]);
			}

			if (isset($_SERVER[$k]) && filter_var($_SERVER[$k], FILTER_VALIDATE_IP))
				return $_SERVER[$k];
		}

		return - 1;
	}

	function getIpInfo ($ip)
	{
		if (filter_var($ip, FILTER_VALIDATE_IP) == false)
			return null;

		if (substr($ip, 0, 3) == "127")
			return null;

		$server = array();
	
		$server_props = array(
				'num' => 1,
				'transport' => 'tcp://',
				'url' => "http://www.geoplugin.net/json.gp?ip=" . $ip,
				'host' => 'geoplugin.net',
				'json' => true,
				'port' => 80
		);
		array_push($server, $server_props);

		$server_props = array(
				'num' => 2,
				'transport' => 'tcp://',
				'url' => "http://ip-api.com/json/" . $ip,
				'host' => 'ip-api.com',
				'json' => true,
				'port' => 80
		);
		array_push($server, $server_props);

		$i = 0;
		$found = false;

		if (count($server) > 1)
		{
			$key = rand(0, count($server) - 1);
			while ($key > 0)
			{
				$temp = array_shift($server);
				$server[] = $temp;
				$key --;
			}
		}

		while ($i < count($server) && $found == false)
		{
			$info = null;

			$error = false;
			$ch = curl_init();

			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
			curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 0);
			curl_setopt($ch, CURLOPT_TIMEOUT, 3);
			curl_setopt($ch, CURLOPT_PORT, $server[$i]['port']);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, true);
			curl_setopt($ch, CURLOPT_URL, $server[$i]['url']);

			$line = curl_exec($ch);
			if ($line === false)
				$error = true;
			curl_close($ch);

			$tmp = json_decode($line, true);

			if ($server[$i]['num'] == 1 && $error == false && $tmp['geoplugin_status'] == 200)
			{
				$info['city']['name'] = $tmp['geoplugin_city'];
				$info['city']['lat'] = $tmp['geoplugin_latitude'];
				$info['city']['lon'] = $tmp['geoplugin_longitude'];
				$info['city']['tag'] = strtolower($tmp['geoplugin_countryCode']);
				$info['server']['url'] = $server[$i]['host'];
			}
			elseif ($server[$i]['num'] == 2 && $error == false && $tmp['status'] == 'success')
			{
				$info['city']['name'] = $tmp['city'];
				$info['city']['lat'] = $tmp['lat'];
				$info['city']['lon'] = $tmp['lon'];
				$info['city']['tag'] = strtolower($tmp['countryCode']);
				$info['server']['url'] = $server[$i]['host'];
			}

			if (isset($info['city']['name']))
			{
				$found = true;
				$i = count($server) + 1;
				return $info;
			}

			$i ++;
		}

		return $info;
	}
	
	function toPrettyString ($json)
	{
		return $this->my_json_format($json);
	}
	
	private function my_json_format ($json)
	{
		$tab = "  ";
		$new_json = "";
		$indent_level = 0;
		$in_string = false;
		
		$len = strlen($json);
		
		for ($c = 0; $c < $len; $c ++)
		{
			$char = $json[$c];
			switch ($char)
			{
				case '{':
				case '[':
					if (! $in_string)
					{
						$new_json .= $char . "\n" . str_repeat($tab, $indent_level + 1);
						$indent_level ++;
					}
					else
					{
						$new_json .= $char;
					}
					break;
				case '}':
				case ']':
					if (! $in_string)
					{
						$indent_level --;
						$new_json .= "\n" . str_repeat($tab, $indent_level) . $char;
					}
					else
					{
						$new_json .= $char;
					}
					break;
				case ',':
					if (! $in_string)
					{
						$new_json .= ",\n" . str_repeat($tab, $indent_level);
					}
					else
					{
						$new_json .= $char;
					}
					break;
				case ':':
					if (! $in_string)
					{
						$new_json .= ": ";
					}
					else
					{
						$new_json .= $char;
					}
					break;
				case '"':
					if ($c > 0 && $json[$c - 1] != '\\')
					{
						$in_string = ! $in_string;
					}
				default:
					$new_json .= $char;
					break;
			}
		}
		return $new_json;
	}

	function format_number ($number, $decimal = 0)
	{
		if ($number === null )
          return null;
		if (preg_match('/ch|da|de|el|es|fr|it|nl|pl|pt|tr/i', $this->lang))
			return number_format($number, $decimal, ',', '.');
		elseif (preg_match('/et|cs|fi|nb|sv|uk/i', $this->lang))
			return number_format($number, $decimal, ',', ' ');
		else
			return number_format($number, $decimal, '.', ',');
	}
	
	function cleanCache($params, $id, $file)
	{
		$db = Factory::getContainer()->get('DatabaseDriver');
		$query = $db->getQuery(true);
		
		$parms = $params->toString();
		$query = "update #__modules set params = " . $db->quote($parms) . " where module = 'mod_js_weather' and id = " . $id  . ";";
		
		$db->setQuery($query);
		$db->execute();
		
		if (file_exists($file))
			unlink($file);
	}
	
	function errorMsg ($code, $msg)
	{
		switch ($code)
		{
			case 400:
				return "<b>" . Text::_('ERROR') . " 400</b><br>" . Text::_('MOD_JS_WEATHER_ERROR_400') . ", <b>Response:</b> " . $msg;
				break;
			case 401:
				return "<b>" . Text::_('ERROR') . " 401</b><br>" .  Text::_('MOD_JS_WEATHER_ERROR_401');
				break;
			case 403:
				return "<b>" . Text::_('ERROR') . " 403</b><br>" .  Text::_('MOD_JS_WEATHER_ERROR_403');
				break;
			case 404:
				return "<b>" . Text::_('ERROR') . " 404</b><br>" . Text::sprintf('MOD_JS_WEATHER_ERROR_404', str_replace("q=", "", $this->getLocation())) ;
				break;
			case 429:
				return "<b>" . Text::_('ERROR') . " 429</b><br>" . Text::_('MOD_JS_WEATHER_ERROR_429');
				break;
			default:
				if ($code == null)
					return "<b>" . Text::_('ERROR') . "</b><br>" . Text::_('MOD_JS_WEATHER_ERROR_NAV');
				return "<b>" . Text::_('ERROR') . " " . $code . "</b><br>" . $msg;
		}
	}
}

