<?php
/**
 * @Copyright
 *
 * @package js_weather for Joomla! 4.x
 * @author     Joachim Schmidt {@link http://www.jschmidt-systemberatung.de/}
 * @version Version: 1.0.0 - 15-February-2023
 * @link       Project Site {@link http://www.jschmidt-systemberatung.de/}
 *
 * @license GNU/GPL
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * change activity: 24.02.2023 First build (v1.0.0)
 *                  15.05.2023 Fixed wrong setting unuts to Fahrenheit
 *                  05.02.2024 Support webfonts provided by owfont - symbol font for Open Weather
 *                  27.03.2024 add option to provide country code for locations
 *                  30.03.2024 improve error handling and error messages
 */
defined('_JEXEC') or die('Direct Access to this location is not allowed.');

use Joomla\CMS\Factory;
use Joomla\CMS\Helper\ModuleHelper;
use js_weather\module\js_weatherClass;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Date\Date;
use Joomla\Input;

require_once (dirname(__FILE__) . '/helper.php');
$helper = new js_weatherClass ($params->get('api_key'), false);

$lang = Factory::getApplication()->getLanguage();
$tag = substr($lang->getTag(), 0, 2);
$helper->setLanguage($tag);

$location =  $params->get('location');
$country = $params->get('country');

$helper->setUnits($params->get('units'));

$unit_info = $helper->getUnitInfo( $params->get('units') );

$rc = $helper->setLocation( $location, $params->get('use_ip'), $country );
if ($rc == 0)
{
  echo  Text::sprintf ('MOD_JS_WEATHER_WEATHER_ERROR_LOCATION', $location);
  return;
}

$currentTime = time();

$module = ModuleHelper::getModule('mod_js_weather', $params->get('title'));
if ($module->id == null)
	$module = ModuleHelper::getModule('mod_js_weather');
	
$module_id = $module->id;
$params->set('title', $module->title);

$dir = JPATH_BASE . "/cache/mod_js_weather";
if ($params->get('cachetime') > 0)
{
    if (!file_exists($dir))
       mkdir (JPATH_BASE . '/cache/mod_js_weather', 0755);
}

$file = $dir . "/save_weatherdata." . $module_id;

if ($params->get('cachetime') > 0 && $params->get('clean_cache') == "1" )
{
	$params->set('clean_cache', "");
	$helper->cleanCache($params, $module_id, $file);
}

if ($params->get('cachetime') > 0 && $helper->getUseIP() == false)
{
	
	$cachetime = $params->get('cachetime') * 60;
	
	if (file_exists($file))
		$changedTime = filemtime($file);
	else
		$changedTime = time();
	
	if (! file_exists($file) || ($currentTime - $changedTime) > $cachetime)
	{
		$helper->setID($module_id);
		$weather_info = json_decode($helper->get_weather_data($dir));
	}
	else
	{
		$currentTime = $changedTime;
		$fp = fopen($file, "r");
		$data = fread($fp, filesize($file));
		$weather_info = json_decode($data);
	}
}
else
{
	if (file_exists($file))
	 unlink($file);
	$weather_info = json_decode($helper->get_weather_data());
}

if ($weather_info->cod == 200)
{
	if (isset($weather_info->main->pressure))
		$weather_info->main->pressure = $helper->format_number($weather_info->main->pressure);
	
	if (isset($weather_info->wind->speed))
	{
	   if ( $helper->getUnits() != "imperial")
		 $weather_info->wind->speed = $weather_info->wind->speed * 3.6;
	   $weather_info->wind->speed = $helper->format_number($weather_info->wind->speed, 1);
    }
	
	if (isset($weather_info->visibility))
	{
		if ($helper->getUnits() == "imperial")
		  $weather_info->visibility = $weather_info->visibility * 3.28084;
	    $weather_info->visibility = $helper->format_number($weather_info->visibility);
	}
	
	$description = ucwords($weather_info->weather[0]->description);
	
	$deg = $weather_info->wind->deg;

	$dirnames = [
		text::_("MOD_JS_WEATHER_N"),  text::_("MOD_JS_WEATHER_NE"), text::_("MOD_JS_WEATHER_E"),
		text::_("MOD_JS_WEATHER_SE"), text::_("MOD_JS_WEATHER_S"),  text::_("MOD_JS_WEATHER_SW"),
		text::_("MOD_JS_WEATHER_W"),  text::_("MOD_JS_WEATHER_NW"), text::_("MOD_JS_WEATHER_N") ];
	
	$wind_dir = $dirnames[round($deg*8/360)];
	
	$date = date("jS F, Y", $currentTime);
	$date = new Date($date);
	$date = $date->format(Text::_('MOD_JS_WEATHER_DATE'));
	
	$wa = Factory::getApplication()->getDocument()->getWebAssetManager();
	if ($params->get('weatherfonts') == true)
	{
		$style_url = "media/mod_js_weather/css/owfont-regular.css";
		$wa->registerAndUseStyle("js_weather0", $style_url);
	}
	
	$style_url = "media/mod_js_weather/css/" . $params->get('style_sheet');
	if (!file_exists($style_url))
		$style_url = "media/mod_js_weather/css/js_weather.css";
	 $wa->registerAndUseStyle("js_weather1", $style_url);
}
else
{
	$error_info =  $helper->errorMsg($weather_info->cod, $weather_info->message);
}

if ( $params->get('moduleclass_sfx') !== null)
	$moduleclass_sfx = htmlspecialchars($params->get('moduleclass_sfx'));
else
	$moduleclass_sfx = "";
require ModuleHelper::getLayoutPath('mod_js_weather', $params->get('layout', 'default'));


?>