<?php
/**
 * @Copyright
 *
 * @package js_weather for Joomla! 4.x
 * @author     Joachim Schmidt {@link http://www.jschmidt-systemberatung.de/}
 * @version Version: 1.0.0 - 15-February-2023
 * @link       Project Site {@link http://www.jschmidt-systemberatung.de/}
 *
 * @license GNU/GPL
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * change activity:  15.05.2023 Fixed wrong setting unuts to Fahrenheit
 *                   05.02.2024 Support webfonts provided by owfont - symbol font for Open Weather
 *
 */

defined('_JEXEC') or die('Restricted access'); // no direct access
use Joomla\CMS\Language\Text;

if ($weather_info->cod != 200)
{
 echo $error_info;
 return;
}

?>

<div class="js_weather<?php echo $moduleclass_sfx ?>">
<div id="js_weather_container" class="js_weather_container">
 <h3 style="font-size:1.1em;text-align: center;"><?php echo Text::_('MOD_JS_WEATHER_WEATHER') . " in " . $weather_info->name; ?></h3>
  <div class="js_weather-time">
   <div><?php echo $date . " - " . gmdate("G:i ", $weather_info->dt + $weather_info->timezone); ?></div>
   <div class="js_weather-forecast"><?php echo $description; ?></div>
  </div>
 
 
  <div class="js_weather-forecast">
   <table class="js_weather-table">
   <tr>
   <?php $width=" width=50" ; if ($unit_info['temperature_unit'] == "K")  $width=" width=40" ;?>
   <?php if ($params->get('weatherfonts') == false) { ?>
     <td style="vertical-align: top;"><img <?php echo $width; ?> src="https://openweathermap.org/img/w/<?php echo $weather_info->weather[0]->icon; ?>.png" title="<?php echo $description; ?>"></td>
   <?php   } else  ?>
     <td style="vertical-align: top;" title="<?php echo $description; ?>"><i class="owf owf-<?php echo $weather_info->weather[0]->id;?> owf-3x"></i></td>
   <?php $style = "style='vertical-align: middle;'"; if ($unit_info['temperature_unit'] == "K") $style = "style='vertical-align: middle; font-size: 1.05em; padding-left: 0px;'" ?>
   <td <?php echo $style; ?>> <?php  if ($params->get('temperature_feels'))
     echo round($weather_info->main->feels_like,0);
    else
     echo round($weather_info->main->temp,0) . "°" . $unit_info['temperature_unit'];?>
   </td>
   <?php $font = ""; if ($unit_info['temperature_unit'] == "K") $font = " style='font-size: 0.66em; padding-left: 0px;'";?>
   <td style="vertical-align: middle; padding-left: 2px;"><span <?php echo $font; ?> class="min-temperature"><?php echo round($weather_info->main->temp_min,0) . "°" . $unit_info['temperature_unit']; ?> min</span></td>
   <td style="vertical-align: middle;"><span <?php echo $font; ?> class="min-temperature"><?php echo round($weather_info->main->temp_max,0) . "°" . $unit_info['temperature_unit']; ?> max</span></td>
   </tr>
   </table>
  </div>
   
  
  <?php if ($params->get('sun')) { ?>
  <div>
   <table class="js_weather-table">
    <tr>
    <?php if ($params->get('weatherfonts') == true) { ?>
      <td style="vertical-align: bottom;" title="<?php echo Text::_('MOD_JS_WEATHER_SUNRISE'); ?>"><i class="owf owf-950 owf-2x"></i></td>
    <?php  } else { ?>
      <td style="vertical-align: bottom;"><img src="media/mod_js_weather/images/sunrise.png" title="<?php echo Text::_('MOD_JS_WEATHER_SUNRISE'); ?>" style="width:42px;"></td>
    <?php  }  ?>
       <td style="vertical-align: middle; padding-right: 15px; "><?php echo " " . gmdate("G:i ", $weather_info->sys->sunrise+$weather_info->timezone); ?></td>
     <?php if ($params->get('weatherfonts') == true) { ?>
      <td style="vertical-align: bottom;"  title="<?php echo Text::_('MOD_JS_WEATHER_SUNSET'); ?>"><i class="owf owf-950-n owf-2x"></i></td>
    <?php  } else { ?>
     <td style="vertical-align: bottom;"><img src="media/mod_js_weather/images/sunset.png" title="<?php echo Text::_('MOD_JS_WEATHER_SUNSET'); ?>" style="width:40px;"></td>
    <?php  }  ?>
    <td style="vertical-align: middle;"><?php echo " " . gmdate("G:i ", $weather_info->sys->sunset + $weather_info->timezone); ?></td>
    </tr>
   </table>
  </div>
  <?php } ?>
  
<?php if ( ($params->get('humidity') || $params->get('wind')) || $params->get('pressure') || $params->get('visibility') ) { ?>
 <div class="js_weather-misc">
  <?php if ($params->get('humidity') && isset($weather_info->main->humidity)) { ?>
   <div><?php echo Text::_('MOD_JS_WEATHER_LABEL_HUMIDITY'); ?>: <?php echo $weather_info->main->humidity; ?> %</div>
  <?php } ?>
  <?php if ($params->get('wind') && isset($weather_info->wind->speed)) { ?>
   <div>Wind: <?php echo $weather_info->wind->speed . $unit_info['speed_unit'] . $wind_dir; ?></div>
  <?php } ?>
  <?php if ($params->get('pressure') && isset($weather_info->main->pressure) ) { ?>
   <div><?php echo Text::_('MOD_JS_WEATHER_LABEL_PRESSURE'); ?>: <?php echo $weather_info->main->pressure; ?> hPa</div>
  <?php } ?>
  <?php if ($params->get('visibility') && isset( $weather_info->visibility) ) { ?>
   <div><?php echo Text::_('MOD_JS_WEATHER_LABEL_VISIBILITY'); ?>: <?php echo $weather_info->visibility . $unit_info['visibilty_unit']; ?></div>
  <?php } ?>
  </div>
<?php }  ?>
</div>

</div>





